<?php
/**
* HELIUM V Synchronisation Modul for Magento 1.7.x
*
* Copyright (C) 2013 HELIUM V IT-Solutions GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* According to sec. 7 of the GNU Affero General Public License, version 3,
* the terms of the AGPL are supplemented with the following terms:
*
* "Helium V" and "Helium 5" are registered trademarks of HELIUM V IT-Solutions GmbH.
* The licensing of the Program under the AGPL does not imply a trademark license.
* Therefore any rights, title and interest in our trademarks remain entirely with us.
* If you want to propagate modified versions of the Program under the
* name "Helium V" or "Helium 5", you may only do so if you have a written permission
* by Helium V IT-Solutions GmbH (to acquire a permission please contact
* Helium V IT-Solutions at trademark@heliumv.com).
*      
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* 
* @category Heliumv
* @package  Heliumv_Synchronization
* @author   Maximilian Hehenwarter <developers@heliumv.com>
* @license  http://www.gnu.org/licenses/  Affero General Public License (AGPL 3.0) 
*/

//wenn über backend aufgerufen aus classe Heliumv_Synchronization_AjaxController dann initialisiere object neu
if(get_class($this)=="Heliumv_Synchronization_AjaxController")
{
    //$installer=new Mage_Eav_Model_Entity_Setup('core_setup');
    $installer = new Mage_Catalog_Model_Resource_Eav_Mysql4_Setup();
}
else
{
     //Das Script wird von der in der config.xml angegebenen Setup Klasse ausgeführt
     //@var $installer Mage_Prodct_Model_Resource_Eav_Mysql4_Setup
    $installer = $this;   
}

/*
 * startSetup() schaltet einige automatische Prüfungen in MySQL ab, um Fehler
 * während Datenbank-Änderungen zu vermeiden.
 */
$installer->startSetup();

//erstellt ein neues attribut external_pid für das product object
//wird in der db unter eav_attribut bzw catalog_product_entity_##typ## abgelegt
//zugriff auf external_pid laut Doku als getExternalPid und setExternalPid, siehe p56, siehe auch verschiedene optionen doku p264
//global = 1 bedeutet diese erweiterung gilt für alle shops dieser magento installaltion
$installer->addAttribute('catalog_product', 'external_pid', array(
	'label' => 'External Product-ID',
	'type' => 'varchar',
    'global' => Mage_Catalog_Model_Resource_Eav_Attribute::SCOPE_GLOBAL,
	'required' => 0,
    'note' => 'Unique Article Identifier from ERP System (Artikel_I_ID)'
));


$installer->addAttribute('catalog_product', 'order_unit', array(
    'label' => 'Order Unit',
    'type' => 'varchar',
    'global' => Mage_Catalog_Model_Resource_Eav_Attribute::SCOPE_GLOBAL,
    'required' => 0,
    'note' => 'Order Unit from ERP System, like Stk, mm, liter, etc.'
));

//neues Attribut SyncTs für das product object
//enthält den zeitstempel wann das produkt zuletzt aktualisiert wurde
//wird nicht in GUI angezeigt
//Möglicher Usecase ausser Debuginfos: Auf Basis dieses TS könnte man bei einem Fullsync alle HV Syncartikel selektieren welche aktuell aber nicht aktualisiert wurden und diese deaktivieren
//global = 1 bedeutet diese erweiterung gilt für alle shops dieser magento installaltion
/*$installer->addAttribute('catalog_product', 'sync_ts', array(
    'label' => 'TS last Artikel Sync',
    'type' => 'datetime',
    'is_visible' => 0,
    'global' => Mage_Catalog_Model_Resource_Eav_Attribute::SCOPE_GLOBAL,
    'required' => 0,
    'note' => 'Timestamp when this Article is synced with ERP System for the last time'
)); */

//neues attribute zur syncronisierung/darstellung von hv shop-categorien in magento
$installer->addAttribute('catalog_category', 'external_gid', array(
    'label' => 'External Group-ID',
    'type' => 'varchar',
    'global' => Mage_Catalog_Model_Resource_Eav_Attribute::SCOPE_GLOBAL,
    'required' => 0,
    'note' => 'Unique Category Identifier from ERP System (ShopgroupID)'
));

//neues attribute zur syncronisierung von Kunden von HV mit magento
$installer->addAttribute('customer', 'external_cid', array(
    'label' => 'External Customer-ID',
    'type' => 'varchar',
    'global' => Mage_Catalog_Model_Resource_Eav_Attribute::SCOPE_GLOBAL,
    'required' => 0,
    'note' => 'Unique Customer Identifier from ERP System (KundenID)'
));
//zusätzlich für Darstelkung von CustomerID in Backend benötigt , wird in tabelle customer_form_attribute abgelegt
// -> input kam von http://blog.plan2.net/2011/07/13/magento-1-5-1-customer-erweitern-admin-felder-backend/
Mage::getSingleton('eav/config')
   ->getAttribute('customer', 'external_cid')
   ->setData('used_in_forms', array('adminhtml_customer'))
   ->save();


//neues attribut für Object Kunden Adresse ID
$installer->addAttribute('customer_address', 'external_aid', array(
    'label' => 'External Adress-ID',
    'type' => 'varchar',
    'global' => Mage_Catalog_Model_Resource_Eav_Attribute::SCOPE_GLOBAL,
    'required' => 0,
    'note' => 'Unique Adress Identifier from ERP System (AdressID)'
));
//zusätzlich für Darstelkung von AdressId in Backend und bei checkout (überleitung order-obj) benötigt, wird in tabelle customer_form_attribute abgelegt
Mage::getSingleton('eav/config')
   ->getAttribute('customer_address', 'external_aid')
   ->setData('used_in_forms', array('adminhtml_customer_address'))
   ->save();
   
   
//anlegen des neuen objektes Multithread zur speicherung der unterschiedlichen jobs
//für datenbank bewusst zu der normalen konvention nich das heliumv davor gestellt!!!
$installer->run("
CREATE TABLE IF NOT EXISTS `{$installer->getTable('heliumv_synchronization_scheduler')}` (
    `entity_id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
    `status` SMALLINT(5) NOT NULL,
    `prio` SMALLINT(5) NOT NULL,
    `jobnr` SMALLINT(5) NOT NULL,
    `retry_counter` SMALLINT(5) NOT NULL,
    `ts_changed` timestamp,
    `class` VARCHAR(50) NOT NULL,
    `call` VARCHAR(50) NOT NULL,
    `arguments` VARCHAR(50) NOT NULL,
    PRIMARY KEY  (`entity_id`),
    INDEX (`status`),
    INDEX (`prio`),
    INDEX (`jobnr`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
");

//anlegen des neuen objektes Multithread zur speicherung der unterschiedlichen jobs
//beispiel von http://stackoverflow.com/questions/4315660/alter-table-in-magento-setup-script-without-using-sql
//weitere infos von https://makandracards.com/magento/6209-table-creation-in-magento-1-6-1-11+
/* @var $this Mage_Core_Model_Resource_Setup */
/*$new_table = new Varien_Db_Ddl_Table();
$new_table = $installer->getConnection();
$new_table->newTable($installer->getTable('heliumvsynchronization_multithread'));
$installer->getConnection()->createTable($new_table);
$new_table->addColumn('entity_id',Varien_Db_Ddl_Table::TYPE_INTEGER,null,array('identity' => true,'unsigned' => true,'nullable' => false,'primary'  => true,),'Entity ID');
$new_table->addColumn('entity_id', Varien_Db_Ddl_Table::TYPE_INTEGER, 11,  array('unsigned' => true, 'primary' => true));
$new_table->addColumn('status', Varien_Db_Ddl_Table::TYPE_INTEGER, 2);
$new_table->addColumn('class', Varien_Db_Ddl_Table::TYPE_VARCHAR, 50);
$new_table->addColumn('call', Varien_Db_Ddl_Table::TYPE_VARCHAR, 50);
$new_table->addColumn('ts_created', Varien_Db_Ddl_Table::TYPE_DATETIME);
$new_table->addColumn('ts_started', Varien_Db_Ddl_Table::TYPE_DATETIME);
$new_table->addColumn('ts_finished', Varien_Db_Ddl_Table::TYPE_DATETIME);
$new_table->addIndex('idx_status','status');
$new_table->setOption('type', 'InnoDB');
$new_table->setOption('charset', 'utf8');
$installer->getConnection()->createTable($new_table);*/


//input von http://tweetorials.tumblr.com/post/12523405410/eav-entity-setup-addattribute-vs-installentities
//$installer->installEntities();

//legt entity an
$installer->installEntities(array(
    'scheduler' => array(
        'entity_model' => 'synchronization/scheduler',
        'table'        => $installer->getTable('heliumv_synchronization_scheduler'),
        'attributes'   => array(
            'status'        => array('type' => 'static', 'label' => 'Status, Created=1, Running=2, Finished=3, Error=4'),
            'prio'        => array('type' => 'static', 'label' => 'Priority of this Thread'),
            'jobnr'        => array('type' => 'static', 'label' => 'Thread Number to calculate the maximum amount of Threads'),
            'retry_counter'        => array('type' => 'static', 'label' => 'Retry attemts in case of an processing error'),
            'ts_created'        => array('type' => 'static', 'label' => 'Timestamp Object changed'),
            'class'     => array('type' => 'static', 'label' => 'PHP Class where Call is defined'),
            'call'  => array('type' => 'static', 'label' => 'Call that will be executed'),
            'arguments'  => array('type' => 'static', 'label' => 'Argumnets of the call, single value or serialized array possible'),
        ),
    ),
));   

   



//setze neuen order status Transfer_start und Transfer_complete
$status = Mage::getModel('sales/order_status');

$status->setStatus('transfer_start')->setLabel('Order Transfer to ERP System started')
    ->assignState(Mage_Sales_Model_Order::STATE_PROCESSING)
    ->save();
    
$status->setStatus('transfer_complete')->setLabel('Order Transfer to ERP System completed')
    ->assignState(Mage_Sales_Model_Order::STATE_PROCESSING)
    ->save();
    
$status->setStatus('transfer_error')->setLabel('Order Transfer to ERP System was not possible')
    ->assignState(Mage_Sales_Model_Order::STATE_PROCESSING)
    ->save();

//deaktiviere das feld "State" unter den Adressen als pflichtfeld (db tabelle core_config_data)
$core_config = new Mage_Core_Model_Config();
$core_config ->saveConfig('general/region/display_all', "0", 'default', 0);
$core_config ->saveConfig('general/region/state_required', "", 'default', 0);

//setze customer initil werte
$core_config ->saveConfig('customer/address/street_lines', "0", 'default', 1);  //nur eine zeile für strasse
$core_config ->saveConfig('customer/address/prefix_show', "0", 'default', 'opt');  //prefix bei adress anzeigen
$core_config ->saveConfig('customer/address/prefix_options', "0", 'default', 'Herr;Frau;Firma'); //prefix daten
$core_config ->saveConfig('customer/address/taxvat_show', "0", 'default', 'opt');  //vat als optional anzeigen
$core_config ->saveConfig('cataloginventory/item_options/manage_stock', "0", 'default', 0);  //lagerverwaltung für alle artikel deaktivieren per default



/*
//patchen der DB Routinen von Magento Libs wegen deadlock         
//input kam von http://phpbugs.wordpress.com/2010/07/12/magento-serialization-failure-1213-deadlock-found-when-trying-to-get-lock-try-restarting-transaction/
$lib_path1=Mage::getBaseDir('lib').DIRECTORY_SEPARATOR.'Varien'.DIRECTORY_SEPARATOR.'Db'.DIRECTORY_SEPARATOR.'Adapter'.DIRECTORY_SEPARATOR;
$lib_path2=Mage::getBaseDir('lib').DIRECTORY_SEPARATOR.'Zend'.DIRECTORY_SEPARATOR.'Db'.DIRECTORY_SEPARATOR.'Statement'.DIRECTORY_SEPARATOR;
//sicherung alter files
copy($lib_path1."Mysqli.php",$lib_path1."Mysqli.php.backup_".gmdate("Ymd_His"));
copy($lib_path2."Pdo.php",$lib_path2."Pdo.php.backup_".gmdate("Ymd_His"));
//hinkopieren neuer files
copy($lib_path1."Mysqli.php.new",$lib_path1."Mysqli.php");
copy($lib_path2."Pdo.php.new",$lib_path2."Pdo.php");
*/


/*
 * endSetup() schaltet die Prüfungen wieder ein
 */
$installer->endSetup();